/*!
 \file    tictactoe.cpp
 \brief   Class for managing the TicTacToe game
 \author  Philippe Lucidarme
 \version 1.0
 \date    18 mars 2011
 */


#include "tictactoe.h"
#include <iostream>



// TicTacToe
/*!
    \brief      Constructor of the class TicTacToe. Call the Reset method. It creates and initializes the game
*/
TicTacToe::TicTacToe(void)
{
    Reset();                                                // Reset the game
}

// Reset
/*!
    \brief      Set the game in its initial configuration. Each cell is filled with the empty symbol
*/
void TicTacToe::Reset(void)
{
    for (int i=0;i<9;i++)                                   // For each cell
        Grid[i]=EMPTY;                                      // put the empty symbol inside
    NumMoves=0;                                             // Set the number of moves at zero
}



// DrawGame
/*!
    \brief      Draw the game in text mode (console mode). This method uses cout to draw the game, it assumes that cout can be used.
    \todo       It would be better to return a string
*/
void TicTacToe::DrawGame(void)
{
    char GridScreen[9];                                     // Create a copy of the game
    for (int i=0;i<9;i++)                                   // For each cell...
    {
        switch (Grid[i])                                    // ...analyze the cell
        {
        case 1 :    {GridScreen[i]='X'; break;}             // Cross
        case 2 :    {GridScreen[i]='O'; break;}             // Circle
        default :   {GridScreen[i]=1+i+'0'; break;}         // Empty cell (replace by the number of the cell
    }
    }
                                                            // Show the current game in text mode
    cout <<" "<<GridScreen[0]<< " | "<< GridScreen[1]<< " | "<< GridScreen[2]<<"\n";
    cout <<" "<<GridScreen[3]<< " | "<< GridScreen[4]<< " | "<< GridScreen[5]<<"\n";
    cout <<" "<<GridScreen[6]<< " | "<< GridScreen[7]<< " | "<< GridScreen[8]<<"\n";
}

// CurrentPlayer
/*! \brief      Return the mark of the current player (CROSS or CIRCLE).
                Note that the methods returns a mark even if the game is over.
                To check if the game is over, uses the \ref GameOver method.
    \return     Returns the mark of the current player (CROSS or CIRCLE)
*/
char    TicTacToe::CurrentPlayer(void)
{
    return NumMoves%2+1;
}


// Play
/*!
     \brief Place the mark of the current player on the grid in the cell NumCell if it is possible.
     \param NumCell : ID of the cell where the current player will place its mark
     \return 1 if the move is possible.
     \return 0 if the move is impossible (NumCell do not bellong to [1;9] or the cell is not empty)
  */
char TicTacToe::Play(int NumCell)
{
    if (NumCell<1 || NumCell>9)                             // Check if it is a cell of the grid
        return 0;
    if (Grid[NumCell-1]!=EMPTY)                             // Check if th cell is empty
        return 0;

    Grid[NumCell-1]=NumMoves%2+1;                           // Place the mark in the grid
    History[NumMoves]=NumCell-1;                            // Memorize for history
    NumMoves++;                                             // Increase the number of moves
    return 1;                                               // Success
}

// Undo
/*!
     \brief     Undo the last move.
     \return    Returns the number of moves since the beginning of the game */
int TicTacToe::Undo(void)
{
    if (NumMoves>=0)
    {
        NumMoves--;
        Grid[History[NumMoves]]=0;
    }
    return NumMoves;
}

// GameOver
/*!
     \brief     Analyse the current state of the game.
     \return    0 if the game is not over.
     \return    CROSS if the game is over and the winner is the first player.
     \return    CIRCLE If the game is over and the winner is the second player.
     \return    DRAW if the game is over and draw.
  */
char TicTacToe::GameOver(void)
{
    if (Grid[0]==Grid[1] && Grid[1]==Grid[2] && Grid[0]!=EMPTY) return Grid[0];
    if (Grid[3]==Grid[4] && Grid[4]==Grid[5] && Grid[3]!=EMPTY) return Grid[3];
    if (Grid[6]==Grid[7] && Grid[7]==Grid[8] && Grid[6]!=EMPTY) return Grid[6];
    if (Grid[0]==Grid[3] && Grid[3]==Grid[6] && Grid[0]!=EMPTY) return Grid[0];
    if (Grid[1]==Grid[4] && Grid[4]==Grid[7] && Grid[1]!=EMPTY) return Grid[1];
    if (Grid[2]==Grid[5] && Grid[5]==Grid[8] && Grid[2]!=EMPTY) return Grid[2];
    if (Grid[0]==Grid[4] && Grid[4]==Grid[8] && Grid[0]!=EMPTY) return Grid[0];
    if (Grid[2]==Grid[4] && Grid[4]==Grid[6] && Grid[6]!=EMPTY) return Grid[6];
    if (NumMoves==9) return DRAW;
    return 0;
}
